@@ -12,10 +12,9 @@ class WordItemManager(odm.Manager):
     def for_model(self, model):
         q = self.query()
         if not isclass(model):
-            return q.filter(model_type = model.__class__,
-                            object_id = model.id)
+            return q.filter(model_type=model.__class__, object_id=model.id)
         else:
-            return q.filter(model_type = model)
+            return q.filter(model_type=model)
 
 
 class WordItem(odm.StdModel):
@@ -29,16 +28,24 @@ class WordItem(odm.StdModel):
     def __unicode__(self):
         return self.word
 
-    objects = WordItemManager()
+    manager_class = WordItemManager
 
     class Meta:
         ordering = -odm.autoincrement()
 
-    @property
-    def object(self):
+    def object(self, session):
         '''Instance of :attr:`model_type` with id :attr:`object_id`.'''
-        if not hasattr(self,'_object'):
-            self._object = self.model_type.objects.get(id = self.object_id)
+        if not hasattr(self, '_object'):
+            pkname = self.model_type._meta.pkname()
+            query = session.query(self.model_type).filter(**{pkname: self.object_id})
+            return query.items(callback=self.__set_object)
+        else:
+            return self._object
+
+    def __set_object(self, items):
+        try:
+            self._object = self.get_unique_instance(items)
+        except self.DoesNotExist:
+            self._object = None
         return self._object
-
-
+        
\ No newline at end of file
