@@ -1,5 +1,6 @@
-'''The :mod:`stdnet.backends.redisb.async` implements an asynchronous connector
-for redis-py_. It uses pulsar_ asynchronous framework. To use this connector,
+'''The :mod:`stdnet.backends.redisb.async` module implements an asynchronous
+connector for redis-py_. It uses pulsar_ asynchronous framework.
+To use this connector,
 add ``timeout=0`` to redis :ref:`connection string <connection-string>`::
 
     'redis://127.0.0.1:6378?password=bla&timeout=0'
@@ -10,6 +11,16 @@ Usage::
     
     db = getdb('redis://127.0.0.1:6378?password=bla&timeout=0')
     
+    
+.. _redis_pubsub:
+
+Asynchronous Publish/Subscribe
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: PubSub
+   :members:
+   :member-order: bysource
+    
 '''
 from collections import deque
 from itertools import chain
@@ -32,8 +43,6 @@ from .extensions import get_script, RedisManager, CppRedisManager
 
 class AsyncRedisRequest(object):
     '''Asynchronous Request for redis.'''
-    pubsub_commands = frozenset(('SUBSCRIBE', 'UNSUBSCRIBE',
-                                 'PSUBSCRIBE', 'PUNSUBSCRIBE'))
     def __init__(self, client, connection, timeout, encoding,
                   encoding_errors, command_name, args, parser,
                   raise_on_error=True, on_finished=None,
@@ -48,16 +57,16 @@ class AsyncRedisRequest(object):
         self.raise_on_error = raise_on_error
         self.on_finished = on_finished
         self.release_connection = release_connection
-        self.response = []
-        self.last_response = False
         self.args = args
+        self.last_response = False
         pool = client.connection_pool
-        if client.is_pipeline:
+        if client.is_pipeline:            
+            self.response = []
             self.command = pool.pack_pipeline(args)
-        elif self.command_name:
+            self.args_options = deque(args)
+        else:
             self.command = pool.pack_command(self.command_name, *args)
-            args =(((command_name,), options),)
-        self.args_options = deque(args)
+            self.options = options
         
     @property
     def key(self):
@@ -85,21 +94,31 @@ class AsyncRedisRequest(object):
     def feed(self, data):
         self.parser.feed(data)
         response = self.parser.get()
-        while response is not False:
-            self.last_response = response
-            if self.args_options:
-                args, options = self.args_options.popleft()
+        c = self.client
+        parse = c.parse_response
+        if c.is_pipeline:
+            while response is not False:
+                self.last_response = response
+                args, opts = self.args_options.popleft()
+                if not isinstance(response, Exception):
+                    response = parse(self, args[0], **opts)
+                self.response.append(response)
+                response = self.parser.get()
+            if not self.args_options:
+                return c.on_response(self.response, self.raise_on_error)
             else:
-                args, options = (self.command_name,), {}
-            if not isinstance(response, Exception):
-                response = self.client.parse_response(self, args[0], **options)
-            self.response.append(response)
-            response = self.parser.get()
-        if not self.args_options:
-            return self.client.on_response(self.response, self.raise_on_error)
+                return NOT_DONE
         else:
-            return NOT_DONE
-    
+            result = NOT_DONE
+            while response is not False:
+                self.last_response = result = response
+                if not isinstance(response, Exception):
+                    result = parse(self, self.command_name, **self.options)
+                elif self.raise_on_error:
+                    raise response
+                response = self.parser.get()
+            return result
+        
     
 class RedisProtocol(pulsar.ProtocolConsumer):
     '''An asynchronous pulsar protocol for redis.'''
@@ -207,8 +226,9 @@ class CppAsyncConnectionPool(AsyncConnectionPoolBase, CppRedisManager):
 
         
 class PubSub(pulsar.EventHandler):
-    '''Implements :class:`PubSub` using a redis backend. To listen for
-messages you can bind to the ``on_message`` event::
+    '''Asynchronous Publish/Subscriber handler for redis.
+    
+To listen for messages you can bind to the ``on_message`` event::
 
     from stdnet import getdb
     
@@ -218,8 +238,11 @@ messages you can bind to the ``on_message`` event::
     redis = getdb('redis://122.0.0.1:6379?timeout=0').client
     pubsub = redis.pubsub()
     pubsub.bind_event('on_message', handle_messages)
+    pubsub.subscribe('mychannel')
 '''
     MANY_TIMES_EVENTS = ('on_message',)
+    subscribe_commands = frozenset((b'unsubscribe', b'punsubscribe',
+                                    b'subscribe', b'psubscribe'))
     
     def __init__(self, connection_pool, shard_hint):
         super(PubSub, self).__init__()
@@ -229,26 +252,54 @@ messages you can bind to the ``on_message`` event::
         self._channels = set()
         self._patterns = set()
     
+    @property
+    def channels(self):
+        '''The set of channels this handler is subscribed to.'''
+        return frozenset(self._channels)
+    
+    @property
+    def patterns(self):
+        '''The set of patterns this handler is subscribed to.'''
+        return frozenset(self._patterns)
+    
     @property
     def is_pipeline(self):
         return False
     
     def publish(self, channel, message):
-        return self.execute_command('PUBLISH', channel, message)
+        '''Publish a new ``message`` to a ``channel``.
+        
+This method return a pulsar Deferred which results in the number of Subscribers
+that will receive the message.'''
+        return self.connection_pool.request(self, 'PUBLISH', channel, message)
     
     @async()
     def subscribe(self, *channels):
-        channels, patterns = self._channel_patterns(channels)        
+        '''Subscribe to a list of ``channels`` or ``channel patterns``.
+        
+It returns an asynchronous component which results in the number of channels
+this handler is subscribed to. If this is the first time the method is called by
+this handler, than the :class:`PubSub` starts listening for messages which
+are fired via the ``on_message`` event.'''
+        channels, patterns = self._channel_patterns(channels)
         if channels:
-            yield self._execute('subscribe', *channels)
-            self._channels.update(channels)
+            channels = tuple(set(channels) - self._channels)
+            if channels:
+                yield self._execute('subscribe', *channels)
+                self._channels.update(channels)
         if patterns:
-            yield self._execute('psubscribe', *patterns)
-            self._patterns.update(patterns)
+            patterns = tuple(set(patterns) - self._patterns)
+            if patterns:
+                yield self._execute('psubscribe', *patterns)
+                self._patterns.update(patterns)
         yield self._count_channels() 
     
     @async()
     def unsubscribe(self, *channels):
+        '''Un-subscribe from a list of ``channels`` or ``channel patterns``.
+        
+It returns an asynchronous component which results in the number of channels
+this handler is subscribed to.'''
         channels, patterns = self._channel_patterns(channels)
         if not channels and not patterns:
             if self._channels:
@@ -270,38 +321,34 @@ messages you can bind to the ``on_message`` event::
             
     @async()
     def close(self):
+        '''Stop listening for messages.
+        
+:meth:`unsubscribe` from all :attr:`channels` and :attr:`patterns`
+and close the subscriber connection with redis.'''
         result = yield self.unsubscribe()
         if self.consumer:
             self.consumer.connection.close()
             self.consumer = None
         yield result
     
-    def parse_response(self, connection, command_name):
-        return connection.read_response()
+    def parse_response(self, connection, command_name, **params):
+        '''Callback from the :class:`AsyncRedisRequest`.
         
-    def on_response(self, result, raise_on_error):
-        for response in result:
-            if isinstance(response, Exception) and raise_on_error:
-                raise response
-            elif isinstance(response, list):
-                command = response[0]
-                if command == b'message':
-                    response = response[1:3]
-                    self.fire_event('on_message', response)
-                elif command == b'pmessage':
-                    response = response[2:4]
-                    self.fire_event('on_message', response)
-                elif command == b'unsubscribe' or command == b'punsubscribe':
-                    response = response[2]
+This method is invoked multiple times when new ``results`` are available.'''
+        response = connection.read_response()
+        if isinstance(response, list):
+            command = response[0]
+            if command == b'message':
+                response = response[1:3]
+                self.fire_event('on_message', response)
+            elif command == b'pmessage':
+                response = response[2:4]
+                self.fire_event('on_message', response)
+            elif command in self.subscribe_commands:
+                response = response[2]
         return response
     
-    def execute_command(self, *args, **options):
-        "Execute a command and return a parsed response"
-        try:
-            return self.connection_pool.request(self, *args, **options)
-        except NoScriptError:
-            self.connection_pool.clear_scripts()
-            raise
+    # INTERNALS
         
     def _channel_patterns(self, channels):
         patterns = []
@@ -323,9 +370,10 @@ messages you can bind to the ``on_message`` event::
             req = self.connection_pool._new_request(self, '', ())
             connection = self.connection_pool.get_connection(req)
             self.consumer = self.connection_pool.consumer_factory(connection)
+            # The consumer does not release the connection
             self.consumer.release_connection = False
         on_finished = Deferred()
-        self.execute_command(command, *args, consumer=self.consumer,
-                             on_finished=on_finished)
+        self.connection_pool.request(self, command, *args, consumer\
+                                     =self.consumer, on_finished=on_finished)
         return on_finished
         
\ No newline at end of file
