@@ -5,6 +5,7 @@ from inspect import isclass
 
 from stdnet.utils.exceptions import *
 from stdnet.utils import UnicodeMixin, unique_tuple
+from stdnet.utils.structures import OrderedDict
 
 from .globals import hashmodel, JSPLITTER, orderinginfo
 from .fields import Field, AutoIdField
@@ -16,16 +17,20 @@ __all__ = ['ModelMeta', 'Model', 'ModelBase', 'ModelState',
 
 
 def get_fields(bases, attrs):
-    fields = {}
-    for base in bases:
-        if hasattr(base, '_meta'):
-            fields.update(deepcopy(base._meta.dfields))
     #
+    fields = []
     for name, field in list(attrs.items()):
         if isinstance(field, Field):
-            fields[name] = attrs.pop(name)
+            fields.append((name, attrs.pop(name)))
     #
-    return fields
+    fields = sorted(fields, key=lambda x: x[1].creation_counter)
+    #
+    for base in bases:
+        if hasattr(base, '_meta'):
+            fields = list((name, deepcopy(field)) for name, field\
+                           in base._meta.dfields.items()) + fields
+    #
+    return OrderedDict(fields)
 
 
 def make_app_label(new_class, app_label=None):
@@ -105,13 +110,18 @@ mapper.
 
 .. attribute:: dfields
 
-    dictionary of :class:`Field` instances. It does not include
-    :class:`StructureField`.
+    dictionary of :class:`Field` instances.
 
 .. attribute:: fields
 
     list of all :class:`Field` instances.
 
+.. attribute:: scalarfields
+
+    Ordered list of all :class:`Field` which are not :class:`StructureField`.
+    The order is the same as in the :class:`Model` definition. The :attr:`pk`
+    field is not included.
+    
 .. attribute:: indices
 
     List of :class:`Field` which are indices (:attr:`Field.index` attribute
@@ -196,20 +206,19 @@ mapper.
         
     def load_state(self, obj, state=None, backend=None):
         if state:
-            id, loadedfields, data = state
-            field = self.pk
-            pkvalue = field.to_python(id, backend)
-            setattr(obj, field.attname, pkvalue)
+            pkvalue, loadedfields, data = state
+            pk = self.pk
+            pkvalue = pk.to_python(pkvalue, backend)
+            setattr(obj, pk.attname, pkvalue)
             if loadedfields is not None:
                 loadedfields = tuple(loadedfields)
             obj._loadedfields = loadedfields
             for field in obj.loadedfields():
                 value = field.value_from_data(obj, data)
                 setattr(obj, field.attname, field.to_python(value, backend))
-            pkname = self.pkname()
             if backend or ('__dbdata__' in data and\
-                            data['__dbdata__'][pkname] == pkvalue):
-                obj._dbdata[self.pkname()] = pkvalue
+                            data['__dbdata__'][pk.name] == pkvalue):
+                obj._dbdata[pk.name] = pkvalue
 
     def __repr__(self):
         return self.modelkey
@@ -235,21 +244,19 @@ Return ``True`` if the instance is ready to be saved to database.'''
         #Loop over scalar fields first
         for field, value in instance.fieldvalue_pairs():
             name = field.attname
-            value = instance.set_field_value(field, value)
             try:
-                svalue = field.serialize(value)
-            except FieldValueError as e:
+                svalue = field.set_get_value(instance, value)
+            except Exception as e:
                 errors[name] = str(e)
             else:
                 if (svalue is None or svalue is '') and field.required:
                     errors[name] = "Field '{0}' is required for '{1}'."\
-                                    .format(name,self)
+                                    .format(name, self)
                 else:
                     if isinstance(svalue, dict):
                         data.update(svalue)
-                    else:
-                        if svalue is not None:
-                            data[name] = svalue
+                    elif svalue is not None:
+                        data[name] = svalue
         return len(errors) == 0
 
     def get_sorting(self, sortby, errorClass=None):
@@ -449,14 +456,14 @@ raised when trying to save an invalid instance.'''
 
     def __new__(cls, *args, **kwargs):
         o = super(Model, cls).__new__(cls)
-        pkname = cls._meta.pkname()
-        setattr(o, pkname, kwargs.pop(pkname, None))
+        pkname = cls._meta.pk.name
+        setattr(o, pkname, kwargs.get(pkname))
         o._dbdata = {}
         return o
 
     def __eq__(self, other):
         if other.__class__ == self.__class__:
-            return self.id == other.id
+            return self.pkvalue() == other.pkvalue()
         else:
             return False
 
@@ -479,16 +486,17 @@ otherwise it returns the cached value.'''
         return self._meta.pk.get_value(self)
 
     @classmethod
-    def get_uuid(cls, id):
-        return '%s.%s' % (cls._meta.hash, id)
+    def get_uuid(cls, pk):
+        return '%s.%s' % (cls._meta.hash, pk)
 
     @property
     def uuid(self):
         '''Universally unique identifier for an instance of a :class:`Model`.'''
-        if not self.id:
+        pk = self.pkvalue()
+        if not pk:
             raise self.DoesNotExist(\
                     'Object not saved. Cannot obtain universally unique id')
-        return self.get_uuid(self.id)
+        return self.get_uuid(pk)
 
     def __get_session(self):
         return self._dbdata.get('session')
