@@ -18,10 +18,11 @@ the main class of :mod:`stdnet.odm` module.'''
     _loadedfields = None
 
     def __init__(self, *args, **kwargs):
-        kwargs.pop(self._meta.pkname(), None)
-        for field in self._meta.scalarfields:
-            self.set_field_value(field, kwargs.pop(field.name, None))
-        attributes = self._meta.attributes
+        meta = self._meta
+        kwargs.pop(meta.pk.name, None)
+        for field in meta.scalarfields:
+            field.set_value(self, kwargs.pop(field.name, None))
+        attributes = meta.attributes
         if args:
             N = len(args)
             if N > len(attributes):
@@ -31,7 +32,8 @@ the main class of :mod:`stdnet.odm` module.'''
                 setattr(self, name, value)
         for name in attributes:
             setattr(self, name, kwargs.pop(name, None))
-        raise_kwargs(self, kwargs)
+        if kwargs:
+            raise_kwargs(self, kwargs)
 
     @property
     def has_all_data(self):
@@ -81,11 +83,6 @@ attribute set to ``True`` won't be included.
             if hasattr(self, name):
                 yield field, getattr(self,name)
 
-    def set_field_value(self, field, value):
-        value = field.to_python(value)
-        setattr(self, field.attname, value)
-        return value
-
     def clear_cache_fields(self):
         '''Set cache fields to ``None``. Check :attr:`Field.as_cache`
 for information regarding fields which are considered cache.'''
@@ -140,7 +137,7 @@ If the *exclude_cache* flag is ``True``, fields with :attr:`Field.as_cache`
 attribute set to ``True`` will be excluded.'''
         odict = {}
         for field,value in self.fieldvalue_pairs(exclude_cache=exclude_cache):
-            value = field.serialize(value)
+            value = field.serialise(value)
             if value:
                 odict[field.name] = value
         if 'id' in self._dbdata:
@@ -151,14 +148,14 @@ attribute set to ``True`` will be excluded.'''
         pk = self.pkvalue()
         if pk:
             yield self._meta.pkname(),pk
-            for field,value in self.fieldvalue_pairs(exclude_cache=\
-                                                     exclude_cache):
-                value = field.json_serialize(value)
+            for field, value in self.fieldvalue_pairs(exclude_cache=\
+                                                      exclude_cache):
+                value = field.json_serialise(value)
                 if value not in EMPTYJSON:
-                    yield field.name,value
+                    yield field.name, value
 
     def tojson(self, exclude_cache=True):
-        '''return a JSON serializable dictionary representation.'''
+        '''Return a JSON serialisable dictionary representation.'''
         return dict(self._to_json(exclude_cache))
 
     def load_fields(self, *fields):
@@ -203,6 +200,10 @@ otherwise it returns ``None``.'''
     
     @classmethod
     def from_base64_data(cls, **kwargs):
+        '''Load a :class:`StdModel` from possibly base64encoded data.
+        
+This method is used to load models from data obtained from the :meth:`tojson`
+method.'''
         o = cls()
         meta = cls._meta
         pkname = meta.pkname()
