@@ -1,6 +1,7 @@
 from copy import copy
 from inspect import isgenerator
 from functools import partial
+from collections import Mapping
 
 from stdnet import range_lookups
 from stdnet.utils import JSPLITTER, iteritems, unique_tuple
@@ -12,11 +13,9 @@ from .globals import lookup_value
 __all__ = ['Q', 'QueryBase', 'Query', 'QueryElement', 'EmptyQuery',
            'intersect', 'union', 'difference']
 
+iterables = (tuple, list, set, frozenset, Mapping)
 def iterable(value):
-    if isgenerator(value) or isinstance(value,(tuple,list,set,frozenset)):
-        return True
-    else:
-        return False
+    return isgenerator(value) or isinstance(value, iterables)
 
 
 def update_dictionary(result, extra):
@@ -724,7 +723,7 @@ an exception is raised.
                 if isinstance(v, Q):
                     v = lookup_value('set', v.construct())
                 else:
-                    v = lookup_value('value', field.dumps(v, lookup))
+                    v = lookup_value('value', field.serialise(v, lookup))
                 lookups.append(v)
         #
         return [queryset(self, name=name, underlying=field_lookups[name])\
