@@ -2,7 +2,7 @@ from itertools import chain
 
 from stdnet import session_result, session_data
 from stdnet.utils.async import on_result, async, multi_async
-from stdnet.utils import itervalues
+from stdnet.utils import itervalues, iteritems
 from stdnet.utils.structures import OrderedDict
 from stdnet.utils.exceptions import *
 
@@ -14,6 +14,7 @@ __all__ = ['Session',
            'Manager',
            'LazyProxy',
            'Transaction',
+           'ModelDictionary',
            'withsession']
 
 def is_query(query):
@@ -97,21 +98,21 @@ class SessionModel(object):
 
     @property
     def new(self):
-        ''''The set of all new instances within this ``SessionModel``. This
+        '''The set of all new instances within this ``SessionModel``. This
 instances will be inserted in the database.'''
         return tuple(itervalues(self._new))
 
     @property
     def modified(self):
         '''The set of all modified instances within this ``Session``. This
-instances will'''
+instances will.'''
         return tuple(itervalues(self._modified))
 
     @property
     def deleted(self):
-        '''The set of all instances marked as 'deleted' within this
+        '''The set of all instance pks marked as `deleted` within this
 :class:`Session`.'''
-        return tuple(itervalues(self._deleted))
+        return tuple((p.pkvalue() for p in itervalues(self._deleted)))
 
     @property
     def dirty(self):
@@ -618,23 +619,50 @@ construct."""
         return EmptyQuery(self.manager(model)._meta, self)
 
     @async()
-    def get_or_create(self, model, **kwargs):
-        '''Get an instance of *model* from the internal cache (only if the
-dictionary *kwargs* is of length 1 and has key given by ``id``) or from the
-server. If it the instance is not available, it tries to create one
-from the **kwargs** parameters.
+    def update_or_create(self, model, **kwargs):
+        '''Update or create a new instance of ``model``. This method can raise
+an exception if the ``kwargs`` dictionary contains field data that does not
+validate.
 
 :parameter model: a :class:`StdModel`
 :parameter kwargs: dictionary of parameters.
-:rtype: an instance of  two elements tuple containing the instance and a boolean
+:returns: A two elements tuple containing the instance and a boolean
     indicating if the instance was created or not.
 '''
+        pkname = model._meta.pkname()
+        pk = kwargs.pop(pkname, None)
         query = self.query(model)
-        model = query.model
-        items = yield query.filter(**kwargs).all()
-        try:
-            yield model.get_unique_instance(items), False
-        except model.DoesNotExist:
+        item = None
+        #
+        if pk:
+            # primary key available
+            items = yield query.filter(pkname=pk).all()
+            if items:
+                item = items[0]
+            else:
+                kwargs[pkname] = pk
+        else:
+            params = {}
+            rest = {}
+            fields = model._meta.dfields
+            for field, value in iteritems(kwargs):
+                if field in fields and fields[field].index:
+                    params[field] = value
+                else:
+                    rest[field] = value
+            if params:
+                items = yield query.filter(**params).all()
+                if len(items) == 1:
+                    item = items[0]
+                    kwargs = rest
+        if item:
+            if kwargs:
+                for field, value in iteritems(kwargs):
+                    setattr(item, field, value)
+                item = yield self.add(item)
+            yield item, False
+        #
+        else:
             item = yield self.add(model(**kwargs))
             yield item, True
 
@@ -797,7 +825,7 @@ subclasses.'''
     
 
 class Manager(object):
-    '''before a :class:`StdModel` can be used in conjunction
+    '''Before a :class:`StdModel` can be used in conjunction
 with a :ref:`backend server <db-index>`, a :class:`Manager` must be associated
 with it via a :class:`Router`. Check the
 :ref:`registration tutorial <tutorial-registration>` for further info::
@@ -812,13 +840,13 @@ with it via a :class:`Router`. Check the
     manager = models[MyModel]
 
 Managers are used as :class:`Session` and :class:`Query` factories
-for a given :class:`StdModel`, but they can be customised::
+for a given :class:`StdModel`::
 
     session = router[MyModel].session()
     query = router[MyModel].query()
     
-To customize a manager for a given model, one creates a subclass and add
-additional method::
+A model can specify a :ref:`custom manager <custom-manager>` by
+creating a :class:`Manager` subclass with additional methods::
 
     class MyModelManager(odm.Manager):
     
@@ -881,10 +909,13 @@ so by setting the ``manager_class`` attribute in the :class:`StdModel`::
         return self._read_backend or self._backend
     
     def __getattr__(self, attrname):
-        result = getattr(self.model, attrname)
-        if isinstance(result, LazyProxy):
-            return result.load_from_manager(self)
-        return result
+        if attrname.startswith('__'): #required for copy
+            raise AttributeError
+        else:
+            result = getattr(self.model, attrname)
+            if isinstance(result, LazyProxy):
+                return result.load_from_manager(self)
+            return result
     
     def __str__(self):
         if self.backend:
@@ -894,17 +925,17 @@ so by setting the ``manager_class`` attribute in the :class:`StdModel`::
         else:
             return '{0}({1})'.format(self.__class__.__name__, self._meta)
     __repr__ = __str__
-
-    def session(self, session=None):
-        '''Returns a new :class:`Session`. This is a shortcut for the
-:meth:`Router.session` method.'''
-        return self._router.session()
     
     def __call__(self, *args, **kwargs):
         # The callable method is equivalent of doing self.model() it is just
         # a shurtcut for a better API
         return self.model(*args, **kwargs)
     
+    def session(self, session=None):
+        '''Returns a new :class:`Session`. This is a shortcut for the
+:meth:`Router.session` method.'''
+        return self._router.session()
+    
     def new(self, *args, **kwargs):
         '''Create a new instance of :attr:`model` and commit it to the backend
 server. This a shortcut method for the more verbose::
@@ -912,6 +943,10 @@ server. This a shortcut method for the more verbose::
     instance = manager.session().add(MyModel(**kwargs))
 '''
         return self.session().add(self.model(*args, **kwargs))
+    
+    def update_or_create(self, **kwargs):
+        '''Invokes the :class:`Session.update_or_create` method.'''
+        return self.session().update_or_create(self.model, **kwargs)
 
     def all(self):
         '''Return all instances for this manager.
@@ -964,9 +999,6 @@ a full text search value.'''
     def keys(self):
         return self.session().keys(self.model)
 
-    def get_or_create(self, **kwargs):
-        return self.session().get_or_create(self.model, **kwargs)
-
     def pkvalue(self, instance):
         '''Return the primary key value for ``instance``.'''
         return instance.pkvalue()
