@@ -100,15 +100,3 @@ def unique_tuple(*iterables):
             vals.append(v)
     return tuple(vals)
 
-memory_symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
-memory_size = dict(((s,1 << (i+1)*10) for i,s in enumerate(memory_symbols)))
-
-def convert_bytes(b):
-    '''Convert a number of bytes into a human readable memory usage'''
-    if b is None:
-        return '#NA'
-    for s in reversed(memory_symbols):
-        if b >= memory_size[s]:
-            value = float(b) / memory_size[s]
-            return '%.1f%sB' % (value, s)
-    return "%sB" % b
