@@ -1,5 +1,32 @@
-'''Classes used for encoding and decoding :class:`stdnet.odm.Field` values.'''
+'''Classes used for encoding and decoding :class:`stdnet.odm.Field` values.
+
+
+.. autoclass:: Encoder
+   :members:
+   :member-order: bysource
+
+
+These are all available :class:`Encoder`:
+
+.. autoclass:: NoEncoder
+
+.. autoclass:: Default
+
+.. autoclass:: NumericDefault
+
+.. autoclass:: Bytes
+
+.. autoclass:: Json
+   
+.. autoclass:: PythonPickle
+
+.. autoclass:: DateTimeConverter
+   
+.. autoclass:: DateConverter
+'''
 import json
+import logging
+
 from datetime import datetime, date
 from struct import pack, unpack
 
@@ -10,10 +37,13 @@ from stdnet.utils import JSONDateDecimalEncoder, pickle, \
 
 nan = float('nan')
 
+LOGGER = logging.getLogger('stdnet.encoders')
+
+
 class Encoder(object):
     '''Virtaul class for encoding data in
-:ref:`remote strcutures <model-structures>`. It exposes two methods
-for serializing and loading data to and from the data server.
+:ref:`data structures <model-structures>`. It exposes two methods
+for encoding and decoding data to and from the data server.
 
 .. attribute:: type
 
@@ -21,42 +51,62 @@ for serializing and loading data to and from the data server.
 '''
     type = None
     
-    def dumps(self, x, logger = None):
+    def dumps(self, x):
         '''Serialize data for database'''
         raise NotImplementedError()
     
-    def loads(self, x, logger = None):
+    def loads(self, x):
         '''Unserialize data from database'''
         raise NotImplementedError()
     
+    def require_session(self):
+        '''``True`` if this :class:`Encoder` requires a
+:class:`stdnet.odm.Session`.'''
+        return False
+    
+    def load_iterable(self, iterable, session=None):
+        '''Load an ``iterable``. By default it returns a generator of
+data loaded via the :meth:`loads` method.
+
+:param iterable: an iterable over data to load.
+:param session: Optional :class:`stdnet.odm.Session`.
+:return: an iterable over decoded data.
+'''
+        data = []
+        load = self.loads
+        for v in iterable:
+            data.append(load(v))
+        return data
+    
         
 class Default(Encoder):
-    '''The default unicode encoder'''
+    '''The default unicode encoder. It converts bytes to unicode when loading
+data from the server. Viceversa when sending data.'''
     type = string_type
     
-    def __init__(self, charset = 'utf-8', encoding_errors = 'strict'):
+    def __init__(self, charset='utf-8', encoding_errors='strict'):
         self.charset = charset
         self.encoding_errors = encoding_errors
         
     if ispy3k:
-        def dumps(self, x, logger = None):
+        def dumps(self, x):
             if isinstance(x,bytes):
                 return x
             else:
                 return str(x).encode(self.charset,self.encoding_errors)
             
-        def loads(self, x, logger = None):
+        def loads(self, x):
             if isinstance(x, bytes):
                 return x.decode(self.charset,self.encoding_errors)
             else:
                 return str(x)
     else:  # pragma nocover
-        def dumps(self, x, logger = None):
+        def dumps(self, x):
             if not isinstance(x,unicode):
                 x = str(x)
             return x.encode(self.charset,self.encoding_errors)
             
-        def loads(self, x, logger = None):
+        def loads(self, x):
             if not isinstance(x,unicode):
                 x = str(x)
                 return x.decode(self.charset,self.encoding_errors)
@@ -74,16 +124,19 @@ def safe_number(v):
 
 
 class NumericDefault(Default):
-    
-    def loads(self, x, logger=None):
-        x = super(NumericDefault,self).loads(x,logger)
+    '''It decodes values into unicode unless they are numeric, in which case
+they are decoded as such.'''
+    def loads(self, x):
+        x = super(NumericDefault, self).loads(x)
         return safe_number(x)
         
     
 class Double(Encoder):
+    '''It decodes values into doubles. If the decoding fails it decodes the
+value into ``nan`` (not a number).'''
     type = float
     
-    def loads(self, x, logger=None):
+    def loads(self, x):
         try:
             return float(x)
         except (ValueError, TypeError):
@@ -92,14 +145,14 @@ class Double(Encoder):
     
     
 class Bytes(Encoder):
-    '''The binary unicode encoder'''
+    '''The binary encoder'''
     type = bytes
     
     def __init__(self, charset = 'utf-8', encoding_errors = 'strict'):
         self.charset = charset
         self.encoding_errors = encoding_errors
         
-    def dumps(self, x, logger = None):
+    def dumps(self, x):
         if not isinstance(x,bytes):
             x = x.encode(self.charset,self.encoding_errors)
         return x
@@ -109,10 +162,10 @@ class Bytes(Encoder):
 
 class NoEncoder(Encoder):
     '''A dummy encoder class'''
-    def dumps(self, x, logger = None):
+    def dumps(self, x):
         return x
     
-    def loads(self, x, logger = None):
+    def loads(self, x):
         return x
     
     
@@ -124,23 +177,21 @@ between python 2 and python 3.'''
     def __init__(self, protocol = 2):
         self.protocol = protocol
         
-    def dumps(self, x, logger = None):
+    def dumps(self, x):
         if x is not None:
             try:
                 return pickle.dumps(x,self.protocol)
             except:
-                if logger:
-                    logger.error('Could not serialize {0}'.format(x),
-                                 exc_info = True)
+                LOGGER.exception('Could not pickle %s', x)
     
-    def loads(self, x, logger = None):
+    def loads(self, x):
         if x is None:
             return x
         elif isinstance(x, bytes):
             try:
                 return pickle.loads(x)
-            except (pickle.UnpicklingError,EOFError,ValueError):
-                return x.decode('utf-8','ignore')
+            except (pickle.UnpicklingError, EOFError, ValueError):
+                return x.decode('utf-8', 'ignore')
         else:
             return x
     
@@ -157,32 +208,31 @@ remote data structures.'''
         self.json_encoder = json_encoder or JSONDateDecimalEncoder
         self.object_hook = object_hook or DefaultJSONHook
         
-    def dumps(self, x, logger = None):
+    def dumps(self, x):
         return json.dumps(x, cls=self.json_encoder)
-        #return s.encode(self.charset, self.encoding_errors)
     
-    def loads(self, x, logger = None):
-        if isinstance(x,bytes):
+    def loads(self, x):
+        if isinstance(x, bytes):
             x = x.decode(self.charset, self.encoding_errors)
-        return json.loads(x, object_hook = self.object_hook)
+        return json.loads(x, object_hook=self.object_hook)
 
 
 class DateTimeConverter(Encoder):
-    '''Convert to and from datetime.datetime and unix timestamps'''
+    '''Convert to and from python ``datetime`` objects and unix timestamps'''
     type = datetime
     
-    def dumps(self, value, logger = None):
+    def dumps(self, value):
         return date2timestamp(value)
     
-    def loads(self, value, logger = None):
+    def loads(self, value):
         return timestamp2date(value)
     
 
 class DateConverter(DateTimeConverter):
     type = date
-    '''Convert to and from datetime.date and unix timestamps'''
+    '''Convert to and from python ``date`` objects and unix timestamps'''
     
-    def loads(self, value, logger = None):
+    def loads(self, value):
         return timestamp2date(value).date()
     
     
